# Description

The `benchmark/live` tools aims to provide a way to find out the maximum number of streams that a targeted Ubicast Live solution can handle before it shows anomalies.
An anomalie is considered to be an m3u8 playlist that has taken more than 4s to update on the Live server.

# Prerequisites

To be able to use the script, you must:
* Be able to directly join the live server defined in the configuration through SSH (the script uses the equivalent of `ssh -l root <LIVE_IP value in the config file>`)
* Be able to directly join the MediaServer defined in the configuration through SSH (the script uses the equivalent of `ssh -l root <MS_IP value in the config file>`)
* The live server and the mediaserver should have the latest version of the Ubicast envsetup repository in the `/root` folder
* The mediaserver should be able to reach the live server through the live server "server_name" and through tcp/80, tcp/443 and tcp/1935 (used to bench the theorical network bandwidth between the servers)
* Have the following debian packages installed on the computer running the bench script: `ksh` (for the nmon graph tool `lib/nmonchart`)
* Have the following debian packages installed on the MediaServer and the Live server: `nmon`
* Make sure that there are no lives currently started and that the MediaServer is not used

# Usage

## Global configuration

Before beeing able to make a bench, copy the example configuration file

```
cd <envsetup_dir>/tools/benchmark/live
cp config.example config
```

Then open the `config` file with a text editor and adapt the variables with your values.

## Single number of streams benchmark

Usage
```
./bench-live.sh [OPTS] STREAM_NUMBER

OPTIONS:
   -t --test-duration    INT  test duration in seconds (default: 30)
   -l --log-file         PATH path for the log file (by default a log is created in the script "output" dir)
   -n --no-header             omit the servers hardware informations
   -m --nmon-monitoring       launch nmon on Mediaserver and Live server
   -h --help                  show this help
```

## Multiple number of streams benchmark

Usage
```
./full-bench.sh [OPTS] -- STREAMS_TARGETS1 STREAMS_TARGET2 ...

OPTIONS:
   -t --test-duration  INT  test duration in seconds (default: 300)
   -h --help                show this help

STREAMS_TARGETS is a list (space separated), consisting of numbers of streams you want to benchmark.
For example, if you want to benchmark the solution for 100, 200, 300 and 400 streams:
./full-bench.sh -- 100 200 300 400

You can also use bash string extention, like this
./full-bench.sh -- {1,2,3,4}00
```

Note: The automatic bench is setup to leave if 10 or more anomalies are detected with the current number of streams

# Result analyzing

The output of the bench can be consulted in the `output` directory of the tool.
Depending on the arguments provided during the bench you can find several files in the corresponding `output` subfolder:
* `sysinfos.txt`: informations about the systems that are being benched (streams quality used, servers CPU/RAM informations, theorical bandwidth measurement)
* `bench.log`: log file of the bench, you can find informations about the execution of the bench in this file. There is also some metrics not provided by the graphs in this file (load average before/after and size of the HLS folder on the live server, number of anomalies detected)
* `bench-ms.nmon` and `bench-live.nmon`: nmon metrics databases for the benchmark
* `graph-ms.html` and `graph-live.html`: HTML pages presenting graphs from the nmon metrics (independent from the nmon DB files)

You can consult the HTML pages `graph-ms.html` and `graph-live.html` by publishing them with a simple static web server. For example by running `python3 -m http.server` in the benchmark output directory and then connecting on the port 8000 with a web browser.
